"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JsonDiff = void 0;
const n8n_workflow_1 = require("n8n-workflow");
const json_diff_1 = require("json-diff");
const lodash_1 = require("lodash");
class JsonDiff {
    constructor() {
        this.description = {
            displayName: 'Json Diff',
            name: 'jsonDiff',
            group: ['transform'],
            version: 1,
            description: 'Json Diff Node',
            icon: 'file:jsondiff.svg',
            defaults: {
                name: 'Json Diff',
            },
            inputs: ['main'],
            outputs: ['main'],
            properties: [
                {
                    displayName: 'Json A',
                    name: 'jsonA',
                    type: 'string',
                    default: '',
                    description: 'Use the expression to add a JSON object to compare',
                },
                {
                    displayName: 'Json B',
                    name: 'jsonB',
                    type: 'string',
                    default: '',
                    description: 'Use the expression to add a JSON object to compare',
                },
                {
                    displayName: 'Check Specific Array',
                    name: 'checkArray',
                    type: 'boolean',
                    default: false,
                    description: 'Whether or not to check a specific array or the whole object',
                },
                {
                    displayName: 'Array to Check',
                    name: 'arrayName',
                    type: 'string',
                    default: '',
                    description: 'The name of the array to check',
                    displayOptions: {
                        show: {
                            checkArray: [true],
                        },
                    },
                },
                {
                    displayName: 'ID Field',
                    name: 'idField',
                    type: 'string',
                    default: '',
                    description: 'Give the ID field you want to use for reference of the differences',
                    displayOptions: {
                        show: {
                            checkArray: [true],
                        },
                    },
                },
            ],
        };
    }
    async execute() {
        const items = this.getInputData();
        let item;
        let jsonA;
        let jsonB;
        const checkArray = this.getNodeParameter('checkArray', 0, false);
        if (checkArray) {
            const arrayName = this.getNodeParameter('arrayName', 0, '');
            const idField = this.getNodeParameter('idField', 0, '');
            const jsonA = this.getNodeParameter('jsonA', 0, {});
            const jsonB = this.getNodeParameter('jsonB', 0, {});
            const arrayA = (0, lodash_1.get)(await getJson(jsonA), arrayName) || [];
            const arrayB = (0, lodash_1.get)(await getJson(jsonB), arrayName) || [];
            if (arrayA.length > 0 && arrayB.length > 0) {
                const sortedArrayA = arrayA.sort((firstItem, secondItem) => firstItem[idField] - secondItem[idField]);
                const sortedArrayB = arrayB.sort((firstItem, secondItem) => firstItem[idField] - secondItem[idField]);
                const difference = (0, json_diff_1.diff)(sortedArrayA, sortedArrayB);
                const differenceWithoutAdditions = difference.filter((x) => x[0] !== '+');
                const additions = difference.filter((x) => x[0] === '+');
                const returnData = [];
                for (let i = 0; i < differenceWithoutAdditions.length; i++) {
                    if (differenceWithoutAdditions[i].length > 1) {
                        const origineel = arrayA[i];
                        const id = origineel[idField];
                        returnData.push({ json: {
                                id: id,
                                diffType: differenceWithoutAdditions[i][0],
                                diff: differenceWithoutAdditions[i][1]
                            } });
                    }
                }
                for (let i = 0; i < additions.length; i++) {
                    if (additions[i].length > 1) {
                        const id = additions[i][1][idField];
                        returnData.push({ json: {
                                id: id,
                                diffType: '+',
                                diff: additions[i][1]
                            } });
                    }
                }
                return [returnData];
            }
            else {
                throw new n8n_workflow_1.NodeOperationError(this.getNode(), 'no Array found');
            }
        }
        else {
            for (let itemIndex = 0; itemIndex < items.length; itemIndex++) {
                try {
                    jsonA = this.getNodeParameter('jsonA', itemIndex, '');
                    jsonB = this.getNodeParameter('jsonB', itemIndex, '');
                    const difference = await compareJson(jsonA, jsonB);
                    item = items[itemIndex];
                    item.json = { diff: difference };
                }
                catch (error) {
                    if (this.continueOnFail()) {
                        items.push({ json: this.getInputData(itemIndex)[0].json, error, pairedItem: itemIndex });
                    }
                    else {
                        if (error.context) {
                            error.context.itemIndex = itemIndex;
                            throw error;
                        }
                        throw new n8n_workflow_1.NodeOperationError(this.getNode(), error, {
                            itemIndex,
                        });
                    }
                }
            }
        }
        return this.prepareOutputData(items);
    }
}
exports.JsonDiff = JsonDiff;
async function compareJson(jsonA, jsonB) {
    let objectJsonA = {};
    let objectJsonB = {};
    try {
        objectJsonA = JSON.parse(jsonA);
    }
    catch {
        objectJsonA = jsonA;
    }
    try {
        objectJsonB = JSON.parse(jsonB);
    }
    catch {
        objectJsonB = jsonB;
    }
    const difference = (0, json_diff_1.diff)(objectJsonA, objectJsonB);
    return difference;
}
async function getJson(json) {
    let objectJson = {};
    try {
        objectJson = JSON.parse(json);
    }
    catch {
        objectJson = json;
    }
    return objectJson;
}
//# sourceMappingURL=JsonDiff.node.js.map